/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "fram2.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define FRAM2_DUMMY 0

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void fram2_cfg_setup ( fram2_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    // Additional gpio pins

    cfg->wp = HAL_PIN_NC;
    cfg->hld = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

FRAM2_RETVAL fram2_init ( fram2_t *ctx, fram2_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return FRAM2_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, FRAM2_DUMMY );

    // Output pins 
    
    digital_out_init( &ctx->wp, cfg->wp );
    digital_out_init( &ctx->hld, cfg->hld );
    digital_out_high( &ctx->hld );

    return FRAM2_OK;
}

void fram2_default_cfg ( fram2_t *ctx )
{
    fram2_write_enable( ctx ); 
}

void fram2_generic_transfer ( fram2_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start(&ctx->spi);
    spi_master_transfer(&ctx->spi, block);
    spi_master_stop(&ctx->spi);    
}

void fram2_set_hold_pin( fram2_t *ctx, uint8_t state )
{
    digital_out_write(&ctx->hld, state);
}

void fram2_write_enable( fram2_t *ctx ) 
{
    spi_master_transfer_data_t block;
    uint8_t temp[1];
    
    temp[0] = FRAM_WREN;

    block.tx_buffer = temp;
    block.tx_length = 1;
    block.rx_buffer = 0;
    block.rx_length = 0;

    fram2_generic_transfer(ctx, &block);  
}

void fram2_write_disable ( fram2_t *ctx )  
{
    spi_master_transfer_data_t block;
    uint8_t temp[1];

    temp[0] = FRAM_WRDI;
    fram2_write_enable(ctx);

    block.tx_buffer = temp;
    block.tx_length = 1;
    block.rx_buffer = 0;
    block.rx_length = 0;

    fram2_generic_transfer(ctx, &block);
}

uint8_t fram2_read_status ( fram2_t *ctx ) 
{
    spi_master_transfer_data_t block;
    uint8_t temp[ 1 ];
    uint8_t receive_buf[ 2 ];

    temp[ 0 ] = FRAM_RDSR;
    
    block.tx_buffer = temp;
    block.tx_length = 1;
    block.rx_buffer = receive_buf;
    block.rx_length = 2;
    
    fram2_generic_transfer(ctx, &block);

    return receive_buf[ 1 ];
}

void fram2_write_status ( fram2_t *ctx, uint8_t value ) 
{
    spi_master_transfer_data_t block;
    uint8_t temp[ 2 ];

    temp[ 0 ] = FRAM_WRSR;
    temp[ 1 ] = value;

    block.tx_buffer = temp;
    block.tx_length = 2;
    block.rx_buffer = 0;
    block.rx_length = 0;

    fram2_write_enable( ctx );
    fram2_generic_transfer( ctx, &block );
}

void fram2_read( fram2_t *ctx, uint32_t address, uint8_t *buffer, uint8_t count ) 
{
    spi_master_transfer_data_t block;
    uint8_t temp[ 4 ];
    uint8_t receive_buf[ 260 ] = { 0 };
    uint8_t i;
    uint8_t k;
    
    temp[ 0 ] = FRAM_READ;
    temp[ 1 ] = (uint8_t)( address >> 16 );
    temp[ 2 ] = (uint8_t)( address >> 8 );
    temp[ 3 ] = (uint8_t)( address & 0x000000FF );
    
    block.tx_buffer = temp;
    block.tx_length = 4;
    block.rx_buffer = receive_buf;
    block.rx_length = count + 4;

    fram2_generic_transfer( ctx, &block );

    k = 4;

    for ( i = 0;  i < count; i++ )
    {
        buffer[ i ] = receive_buf[ k++ ];
    }
}

void fram2_write ( fram2_t *ctx, uint32_t address, uint8_t *buffer, uint8_t counter ) 
{
    spi_master_transfer_data_t block;
    uint8_t temp[260];
    uint8_t i;

    temp[ 0 ] = FRAM_WRITE;
    temp[ 1 ] = (uint8_t)( address >> 16 );
    temp[ 2 ] = (uint8_t)( address >> 8 );
    temp[ 3 ] = (uint8_t)( address & 0x000000FF );
                           
    for ( i = 4; i < counter + 4; i++ )
    {
        temp[ i ] = *buffer;
        buffer++;
    }

    block.tx_buffer = temp;
    block.tx_length = counter + 4;
    block.rx_buffer = 0;
    block.rx_length = 0;

    fram2_write_enable(ctx);
    fram2_generic_transfer(ctx, &block);
}

// ------------------------------------------------------------------------- END

